/**
 * @module
 * @license MIT
 * @see {@link https://www.npmjs.com/package/yaml-lint|YAML Lint}
 * @author Sébastien Règne
 */

import yamlLint from "yaml-lint";
import SEVERITY from "../severity.js";

/**
 * @typedef {import("../../types").Notice} Notice
 */

/**
 * Vérifie un fichier avec le linter <strong>YAML Lint</strong>.
 *
 * @param {string}           file          Le fichier qui sera vérifié.
 * @param {Object|undefined} options       Les options qui seront passées au
 *                                         linter ou <code>undefined</code> pour
 *                                         les options par défaut.
 * @param {Object}           context       Le contexte avec d'autres
 *                                         informations.
 * @param {number}           context.level Le niveau de sévérité minimum des
 *                                         notifications retournées.
 * @returns {Promise<Notice[]>} Une promesse retournant la liste des
 *                              notifications.
 */
export const wrapper = async function (file, options, { level }) {
    if (SEVERITY.ERROR > level) {
        return [];
    }

    try {
        await yamlLint.lintFile(file, options);
        return [];
    } catch (err) {
        return [
            {
                file,
                linter: "yaml-lint",
                severity: SEVERITY.ERROR,
                message: err.reason,
                locations: [
                    {
                        // Augmenter de un le numéro de la ligne et de la
                        // colonne car YAML Lint commence les numérotations à
                        // zéro.
                        line: err.mark.line + 1,
                        column: err.mark.column + 1,
                    },
                ],
            },
        ];
    }
};
